//***************************************************************************//
//* FRIDA: fast reliable interactive data analysis                          *//
//* (C) Joachim Wuttke 2001-                                                *//
//* http://apps.jcns.fz-juelich.de/frida                                    *//
//***************************************************************************//

//! \file  func.hpp
//! \brief Collection NFunctions of function wrappers CFunc.

//  Low-level function types.

typedef double (*func_f1) (double);
typedef double (*func_f2) (double, double);
typedef double (*func_f3) (double, double, double);

//  Derivatives thereof.

typedef void (*deri_f1) (double&,double&,double,double);
typedef void (*deri_f2) (double&,double&,double,double,double,double);
typedef void (*deri_f3) (double&,double&,
                         double,double,double,double,double,double);

//! A wrapper holding a function, its gradient, its name, and more.

class CFunc {
 public:
    int narg;
    string txt;
    int prec;
    string com;
    CFunc( int _narg, string _txt, int _prec, string _com="" )
        : narg(_narg), txt(_txt), prec(_prec), com(_com) {};
    virtual ~CFunc() {}; // magic: makes dynamic_cast possible

    void register_me() const;
};

//! Wrapping a function with 1 argument.

class CFunc1 : public CFunc {
 public:
    func_f1 f1;
    deri_f1 d1;
    CFunc1( string _txt, func_f1 _f, deri_f1 _d, int _prec, string _com="" )
        : CFunc( 1, _txt, _prec, _com ), f1(_f), d1(_d) {};
    class CCoord coord( class CCoord *co ) const;
};

//! Wrapping a function with 2 arguments.

class CFunc2 : public CFunc {
 public:
    func_f2 f2;
    deri_f2 d2;
    CFunc2( string _txt, func_f2 _f, deri_f2 _d, int _prec, string _com="" )
        : CFunc( 2, _txt, _prec, _com ), f2(_f), d2(_d) {};
    class CCoord coord( class CCoord *co1, class CCoord *co2 ) const;
};

//! Wrapping a function with 3 arguments.

class CFunc3 : public CFunc {
 public:
    func_f3 f3;
    deri_f3 d3;
    CFunc3( string _txt, func_f3 _f, deri_f3 _d, int _prec, string _com="" )
        : CFunc( 3, _txt, _prec, _com ), f3(_f), d3(_d) {};
    class CCoord coord(
        class CCoord *co1, class CCoord *co2, class CCoord *co3 ) const;
};

//! Collection of function wrappers CFunc

namespace NFunctions { // public (short) interface
    void initialize();
    const CFunc* find( string nam );
    void display_functions();
}
